package puppet;


import java.util.HashSet;
import java.util.Iterator;

import org.ggf.schemas.graap.x2005.x09.wsAgreement.AgreementOfferDocument;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.AgreementType;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.TermTreeType; 

import constant.Constant;

import puppet.ambition.Fuppet;
import puppet.commonFeatures.CommonFeatures;
import puppet.mobility.MovingPuppet;



public class Parse_Wsa 
{
	private String agreementFilename;
	private String gtTrueListFilename;
    private String qcMappingFilename;
    private String sloMappingFilename;
    private String ambitionMode;
    private String mobilityMode;
    
	public Parse_Wsa(String agreementFilename, String gtTrueListFilename, String qcMappingFilename, String sloMappingFilename, String ambitionMode, String mobilityMode)
	{
	    this.agreementFilename = agreementFilename;
        this.gtTrueListFilename = gtTrueListFilename;
        this.qcMappingFilename = qcMappingFilename;
        this.sloMappingFilename = sloMappingFilename;
        this.ambitionMode = ambitionMode;
        this.mobilityMode = mobilityMode;
    }

	public void wsa_parser( String WSDD_File, String wsdlFile, String Destination)
	{
		/* filename
		 * QCFile path del file  contenente il mapping del QualifyngCondition
		 * SLOFile path del file contenente il mapping del ServiceLevelObjective
		 * Servizio nome del servizio
		 */
		// Carico il File Agreement e comincio a parsare la root
		Support Obj_support = new Support();
		AgreementOfferDocument puppetDoc = Obj_support.getAgreementOffer(this.agreementFilename);
		WSAManipulator WSA_obj = new WSAManipulator(this.qcMappingFilename);
		ServiceLevelObjectiveManipulator SLO_obj = new ServiceLevelObjectiveManipulator(this.sloMappingFilename, ambitionMode);
		TermTreeType termTree=null;
		AgreementType agrType = null;		
		try
		{
			//parso il tag AgreementOffer
			agrType=puppetDoc.getAgreementOffer();
			System.out.println("AgreementOffer");
			termTree=agrType.getTerms();
			System.out.println("getTerms ");
			BuilderLogicGTT ElencoGTT = new BuilderLogicGTT(this.gtTrueListFilename);
			TermParser term_parser = new TermParser(WSDD_File,Destination,WSA_obj, SLO_obj, this.qcMappingFilename);
			System.out.println("RecursiveTermParser");
			term_parser.RecursiveTermParser(ElencoGTT.GetElenco(), termTree);

			if (ambitionMode.equals(Constant.AMBITION_MODE_ON)){
			    System.out.println("Including Ambition");
			    insertAmbitionPart(term_parser,wsdlFile);
			}

            if (mobilityMode.equals(Constant.MOBILITY_MODE_ON)){
                System.out.println("Including Mobility Emulation");
                insertMovingPart(term_parser,wsdlFile);
            }
            
//            boolean isThereAnyCommonFeatureToAdd = ambitionMode.equals(Constant.AMBITION_MODE_ON) || mobilityMode.equals(Constant.MOBILITY_MODE_ON); 
            boolean isThereAnyCommonFeatureToAdd = true; 
            if (isThereAnyCommonFeatureToAdd)
                insertCommonFeaturesPart(term_parser,wsdlFile);
            
		}
		catch(Exception e){
e.printStackTrace();
            //            System.out.println(e);
            }
	}

    /*
     * Insert the behavior that is common to any selected features, 
     * e.g. the computation of the time elapsed for the emulation of the 
     * functional/mobility part   
     */
    private void insertCommonFeaturesPart(TermParser termParser, String wsdlFile) {
        HashSet<String> serviceList;
        Iterator<String> list;
        CommonFeatures wrapper;
        String serviceLocation;

        serviceList = termParser.getServiceList();
        list = serviceList.iterator();
        
        while(list.hasNext()){
          serviceLocation = list.next();
          wrapper = new CommonFeatures(serviceLocation, wsdlFile);
          wrapper.parseAndDump();            
        }        
    }

    /*
     * Insert the emulation of functional aspects into the generated stubs
     */
    private void insertAmbitionPart (TermParser termParser, String wsdlFile){       
        HashSet<String> serviceList;
        Iterator<String> list;
        Fuppet wrapper;
        String serviceLocation;

        serviceList = termParser.getServiceList();
        list = serviceList.iterator();
        
//int i=0;
        while(list.hasNext()){
          serviceLocation = list.next();
//System.out.println(i + "--" + serviceLocation);
          wrapper = new Fuppet (serviceLocation, wsdlFile);
          wrapper.parseAndDump();            
        }        
    }

    
    /*
     * Insert the emulation of the mobility aspects into the generated stubs
     */
    private void insertMovingPart (TermParser termParser, String wsdlFile){    
        HashSet<String> serviceList;
        Iterator<String> list;
        MovingPuppet wrapper;
        String serviceLocation;

        serviceList = termParser.getServiceList();
        list = serviceList.iterator();
        
//int i=0;
        while(list.hasNext()){
          serviceLocation = list.next();
//System.out.println(i + "--" + serviceLocation);
          wrapper = new MovingPuppet (serviceLocation, wsdlFile);
          wrapper.parseAndDump();            
        }        
    }

}		