package puppet;


import gnu.getopt.Getopt;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import org.ini4j.IniFile;

import constant.Constant;

public class Puppet {

	private String wsdlPath;
	private String targetPath;
    private String wsaFilename;

    private String qcMappingFilename;
    private String sloMappingFilename;
    	
    public Puppet (String wsdlPath, String wsaFilename, String targetPath, String qcMappingFilename, String sloMappingFilename){

        this.wsdlPath = wsdlPath;
        this.targetPath = targetPath;
        this.wsaFilename = wsaFilename;

        this.qcMappingFilename = qcMappingFilename;
        this.sloMappingFilename = sloMappingFilename;

    }

    public Puppet (String wsdlPath, String wsaFileName, String targetPath)
	{	
        this (wsdlPath, wsaFileName, targetPath, Constant.DEAFULT_MAPPED_QC_FILENAME, Constant.DEAFULT_MAPPED_SLO_FILENAME);
	}
	
	public void generatePuppets (String trueTermsFilename, String ambitionMode, String mobilityMode){
        System.out.println("Start");
        
        HashMap<String, String> wsddLocationFromWSDL;
        
        //1.carico da una directory l'elenco di tutti i file wsdl ed eseguo su ognuno il WSDL2Java
        WSDLCreator WC = new WSDLCreator(this.wsdlPath,this.targetPath);
        //Creo gli stub e verifico che tutto sia andato bene
        if(WC.WSDL2Java()){}
        else
        {
            System.out.println("Error creating Java Stubs in WSDL2Java");
        }

        Parse_Wsa parser = new Parse_Wsa(this.wsaFilename, trueTermsFilename,this.qcMappingFilename,this.sloMappingFilename,ambitionMode, mobilityMode);
        
        WSDD_Finder W_F=new WSDD_Finder();
        wsddLocationFromWSDL = W_F.wsdl2wsddLocation(this.wsdlPath);
        Iterator wsdlIterator = wsddLocationFromWSDL.keySet().iterator();
        String wsddFilename;
        String wsdlFilename;
        while (wsdlIterator.hasNext()){
            wsdlFilename = (String) wsdlIterator.next();
            wsddFilename = wsddLocationFromWSDL.get(wsdlFilename)+"deploy.wsdd";
            
            System.out.println("wsdd -> " + this.targetPath+wsddFilename );
            System.out.println("wsdl -> " + this.wsdlPath+wsdlFilename);            
            System.out.println("+++++++++++++++++++++++++++++++++++++++++");
            
            parser.wsa_parser(this.targetPath+wsddFilename,this.wsdlPath+wsdlFilename,this.targetPath);           
        }

        System.out.println("parsing done");        
        
    }
    
    public static void main(String[] args) 
    {

        Getopt g = new Getopt("testprog", args, "h");

        int c;
        String arg;
        while ((c = g.getopt()) != -1)
          {
            switch(c)
              {
                 default:
                     System.out.println("Usage: java puppet.Puppet <IniConfigurationFile>");
                     System.exit(1);            
              }
          }
                
        
        if (args.length != 1){
            System.out.println("Usage: java puppet.Puppet <IniConfigurationFile>");
            System.exit(1);            
        }
        
        Preferences preferenceRef;
        
        String wsdlPathLocal="";        
        String targetPathLocal="";
        String wsaFilenameLocal="";
        String wsaPathLocal="";
        String qcMappingFilenameLocal="";
        String sloMappingFilenameLocal="";
        String trueTermsFilenameLocal="";
        String trueTermsPathLocal="";
        String ambitionMode="";
        String mobilityMode="";
        
        try {
            preferenceRef = new IniFile(new File(args[0]));
            
            wsdlPathLocal = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("wsdlPath", Constant.DEFAULT_GENERIC_PATH);
            targetPathLocal = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("targetPath", Constant.DEFAULT_GENERIC_PATH);

            wsaFilenameLocal = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("wsaFilename", Constant.DEAFULT_AGREEMENT_FILENAME);
            wsaPathLocal = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("wsaPath", Constant.DEFAULT_GENERIC_PATH);
            
            qcMappingFilenameLocal = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("qcMappingFilename", Constant.DEAFULT_MAPPED_QC_FILENAME);
            sloMappingFilenameLocal = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("sloMappingFilename", Constant.DEAFULT_MAPPED_SLO_FILENAME);
            
            trueTermsFilenameLocal = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("trueTermsFilename", Constant.DEAFULT_GT_TRUE_ITEMS_FILENAME);
            trueTermsPathLocal = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("trueTermsPath", wsaPathLocal);
            
            ambitionMode = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("ambitionMode", Constant.DEFAULT_AMBITION_MODE);

            mobilityMode = preferenceRef.node(Constant.REFERENCE_SECTION_IN_INIFILE).get("mobilityMode", Constant.DEFAULT_MOBILITY_MODE);

            Puppet puppetGenerator = new Puppet (wsdlPathLocal, wsaPathLocal+wsaFilenameLocal, targetPathLocal, qcMappingFilenameLocal, sloMappingFilenameLocal);
            puppetGenerator.generatePuppets(trueTermsPathLocal+trueTermsFilenameLocal,ambitionMode,mobilityMode);

        } catch (BackingStoreException e) {
            e.printStackTrace();
        }
    }
    
}
