package puppet.commonFeatures;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.antlr.stringtemplate.StringTemplate;

public class CommonMethodCodeProcessor {

    protected String name;    
    protected String methodType;    
// Are the qualificator such as "private", "public" and so on    
    protected String methodPrefix;
    protected CommonPatternsReader commonPatterns;
    protected CommonVariableWrapper commonVarWrapper;
    private static final String COMMON_PREEFIX_IN_USE="commonPrefix";
    
    public CommonMethodCodeProcessor(String methodName, String methodPrefix) {
        this.methodPrefix = methodPrefix;
        this.methodType = "void";        
        this.commonPatterns = new CommonPatternsReader(CommonMethodCodeProcessor.COMMON_PREEFIX_IN_USE);
        this.commonVarWrapper = new CommonVariableWrapper();
        name = methodName;
    }

    public CommonMethodCodeProcessor(String methodName) {
        this(methodName,"public");
    }
              
    protected boolean isMethodVoid (){
       return this.methodType.equals("void");   
    }
    
    protected String getReturnRegExpr(){        
        if (isMethodVoid())
            return "";
        return "return\\s+[^;]+[;]";   
    }
    
    protected String getMethodRegExpr(){
        if (isMethodVoid())
            return this.name+"\\s*[(][^{]*[{]";
        return this.name+"\\s*[(][^{]*[{][\\p{Punct}\\p{Alnum}\\p{Space}]*?"+getReturnRegExpr()+"\\p{Space}*[}]";    
    }
    
    public String parseCode(String code){
        String tempBuffer;
        String localSourceCodeBuffer = code;
                
        if (this.name != null){
            this.methodType = extractMethodType(code);
            
            String methodRegExpr = getMethodRegExpr(); 

            Pattern methodPattern = Pattern.compile(methodRegExpr, Pattern.MULTILINE);
            Matcher methodDefinitionMatcher = methodPattern.matcher(code);

            while (methodDefinitionMatcher.find()){
                String methodDefinitionString = methodDefinitionMatcher.group();
                String commonTimeStampString = this.commonPatterns.getCurrentTimeStampStatements();

                commonTimeStampString = this.commonPatterns.getCommonVariableCode(this.methodType) + commonTimeStampString; 
                
                if (!methodDefinitionString.contains(methodPrefix) ){
// Add the statements to get the timeStamp at invocation time                    
                    tempBuffer = methodDefinitionString.replaceFirst("\\{", "\\{" + commonTimeStampString); 
                    
                    int start = methodDefinitionMatcher.start();
                    int offset = methodDefinitionMatcher.end();
                                        
                    localSourceCodeBuffer = this.exchangePortion(code, start, offset, tempBuffer);
                    localSourceCodeBuffer = localSourceCodeBuffer.replaceAll("/\\*\\s*LocalComputationEffort\\s*\\*/", this.commonPatterns.getElapsedTimeStatement());
                }
            }
        }

        StringTemplate templateParser = new StringTemplate(localSourceCodeBuffer);
        templateParser.setAttribute(CommonMethodCodeProcessor.COMMON_PREEFIX_IN_USE, this.commonVarWrapper.getCommonPrefix());        
        localSourceCodeBuffer = templateParser.toString();
        
        return localSourceCodeBuffer;
    };

    protected String extractMethodType(String code) {
        String methodRegExpr = this.methodPrefix+"\\p{Space}+\\p{Alpha}[\\p{Alnum}_.]*\\p{Space}+"+this.name+"\\s*[(]"; 

        Pattern methodPattern = Pattern.compile(methodRegExpr, Pattern.MULTILINE);
        Matcher methodDefinitionMatcher = methodPattern.matcher(code);

        String returnedMethodType = "void";
        
// If there is no overloading as we are supposing, it should be executed just once.        
        while (methodDefinitionMatcher.find()){
            String methodDefinitionString = methodDefinitionMatcher.group();
            String [] methodDefinitionParts = methodDefinitionString.split("\\s");            
            returnedMethodType = methodDefinitionParts[1];
        }
        
        return returnedMethodType;
    }

    protected String exchangePortion(String originalString, int beginRegion, int endRegion, String newString){
        int originalStringLength =  originalString.length();
        String exchangedString = originalString.substring(0,beginRegion)+newString+originalString.substring(endRegion, originalStringLength);            
        return exchangedString;
    }
    
    
    public String getName() {
        return name;
    }

    public String getMethodType() {
        return methodType;
    }

}
