package puppet.mobility;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.antlr.stringtemplate.StringTemplate;

public class MethodCodeProcessor4Mobility {

    private String name;
    private String methodType;
    private MobilityVariableWrapper mobilityVarWrapper;
    private PatternsReader4Mobility mobilityPatterns;    
// Are the qualificator such as "private", "public" and so on    
    private String methodPrefix;

    public MethodCodeProcessor4Mobility (String methodName, String methodPrefix) {
        this.methodPrefix = methodPrefix;
        this.methodType = "void";
        this.mobilityVarWrapper = new MobilityVariableWrapper();
        this.mobilityPatterns = new PatternsReader4Mobility();
        this.name = methodName;
    }

    public MethodCodeProcessor4Mobility (String methodName) {
        this(methodName,"public");
    }


    private String getMobilityInvocationCode (String mobilityModelNameOfTheStub){
        String generatedCode="/* Code Generated for Mobility Emulation \n */ \n";
        generatedCode = generatedCode + this.mobilityPatterns.getMobilityOracleInvocationCode();        
        return generatedCode;
    }
    
    private boolean isMethodVoid (){
       return this.methodType.equals("void");   
    }
    
    private String getReturnRegExpr(){        
        if (isMethodVoid())
            return "";
        return "return\\s+[^;]+[;]";   
    }
    
    private String getMethodRegExpr(){
        if (isMethodVoid())
            return this.name+"\\s*[(][^{]*[{]";
        return this.name+"\\s*[(][^{]*[{][\\p{Punct}\\p{Alnum}\\p{Space}]*?"+getReturnRegExpr()+"\\p{Space}*[}]";    
    }
    
    public String parseCode(String code){
        String tempBuffer;
        String localSourceCodeBuffer = code;
                
        if (this.name != null){
            this.methodType = extractMethodType(code);
            
//            parametersList = extractParametersList(code);

            String methodRegExpr = getMethodRegExpr(); 

            Pattern methodPattern = Pattern.compile(methodRegExpr, Pattern.MULTILINE);
            Matcher methodDefinitionMatcher = methodPattern.matcher(code);

            while (methodDefinitionMatcher.find()){
                String mobilityInvoString = this.getMobilityInvocationCode("NOME DEL SERVIZIO NEL FILE DI NS");
                
                String methodDefinitionString = methodDefinitionMatcher.group();
                
                if (!methodDefinitionString.contains(methodPrefix) ){
                  tempBuffer = methodDefinitionString.replaceFirst("\\{", "\\{" + mobilityInvoString); 
                    
                    int start = methodDefinitionMatcher.start();
                    int offset = methodDefinitionMatcher.end();
                                        
                    localSourceCodeBuffer = this.exchangePortion(code, start, offset, tempBuffer);
                }
            }
        }

        StringTemplate templateParser = new StringTemplate(localSourceCodeBuffer);
        templateParser.setAttribute("mobilityPrefix", mobilityVarWrapper.getMobilityPrefix());        
        localSourceCodeBuffer = templateParser.toString();
        
        return localSourceCodeBuffer;
    }

    private String extractMethodType(String code) {
        String methodRegExpr = this.methodPrefix+"\\p{Space}+\\p{Alpha}[\\p{Alnum}_.]*\\p{Space}+"+this.name+"\\s*[(]"; 

        Pattern methodPattern = Pattern.compile(methodRegExpr, Pattern.MULTILINE);
        Matcher methodDefinitionMatcher = methodPattern.matcher(code);

        String returnedMethodType = "void";
        
// If there is no overloading as we are supposing, it should be executed just once.        
        while (methodDefinitionMatcher.find()){
            String methodDefinitionString = methodDefinitionMatcher.group();
            String [] methodDefinitionParts = methodDefinitionString.split("\\s");            
            returnedMethodType = methodDefinitionParts[1];
        }
        
        return returnedMethodType;
    }

    private String exchangePortion(String originalString, int beginRegion, int endRegion, String newString){

        int originalStringLength =  originalString.length();
        String exchangedString = originalString.substring(0,beginRegion)+newString+originalString.substring(endRegion, originalStringLength);            
        return exchangedString;
    }
    
    
    public String getName() {
        return name;
    }

    public String getMethodType() {
        return methodType;
    }
    
}
