package puppet.slang;

import it.cnr.isti.setest0.puppetScope.PuppetScopeDocument;
import it.cnr.isti.setest0.puppetScope.PuppetScopeDocument.PuppetScope;

import org.apache.xmlbeans.XmlObject;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.BusinessValueListType;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.GuaranteeTermType;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.ServiceSelectorType;
import org.ggf.schemas.graap.x2005.x09.wsAgreement.TermCompositorType;
import org.w3c.dom.Node;

import uk.ac.ucl.cs.slangta.jmi.impl.services.es.Operation_Impl;
import uk.ac.ucl.cs.slangta.jmi.impl.slang.SLA_Impl;
import uk.ac.ucl.cs.slangta.jmi.impl.slang.es.ElectronicServiceDefinition_Impl;
import uk.ac.ucl.cs.slangta.jmi.impl.slang.es.ElectronicServiceInterfaceDefinition_Impl;

public abstract class GuaranteeTermGenerator {
    
    protected TermCompositorType compositorElement;
    protected static int guaranteeTermId = 0;
    
    protected SLA_Impl slangSpecification;
    
    
    public GuaranteeTermGenerator (SLA_Impl slangSpecification){
        this.compositorElement = TermCompositorType.Factory.newInstance();
        this.slangSpecification = slangSpecification;
    }
    
    public GuaranteeTermGenerator (SLA_Impl slangSpecification, TermCompositorType compositorElement){
        this.compositorElement = compositorElement;
        this.slangSpecification = slangSpecification;
    }
    
    /*
     * Appends and returns a ServiceSelectorType element to the
     * GuaranteeTermType
     */
    protected ServiceSelectorType generateServiceScope(GuaranteeTermType guaranteeTerm) {
        Node importedNode;
                
        ServiceSelectorType scopeElement = guaranteeTerm.addNewServiceScope();

        
        // FRANCO: An SLA can be applied to more than one Service. Here I take the first
        // service only.
        
        java.util.Set servicesSet = slangSpecification.getServices();
        
        if(servicesSet != null) {
    		
			javax.jmi.reflect.RefObject [] values =
				(javax.jmi.reflect.RefObject [])servicesSet.toArray(
					new javax.jmi.reflect.RefObject[servicesSet.size()]);
        
			ElectronicServiceDefinition_Impl service = (ElectronicServiceDefinition_Impl) values[0];
			

			// FRANCO: the name of the service is its identifier.
			String serviceName = service.getIdentifier();			
			scopeElement.setServiceName(serviceName);
			
			// FRANCO: these are the interfaces of a service. The interfaces are Puppet "operations"
			// (not to be confused with SLAng operations, one level down)
			java.util.Set interfacesSet = service.getInterfaces();
			javax.jmi.reflect.RefObject [] ivalues =
				(javax.jmi.reflect.RefObject [])interfacesSet.toArray(
					new javax.jmi.reflect.RefObject[interfacesSet.size()]);
			
			if ( ivalues != null ) {
		        PuppetScopeDocument pScopeDoc = PuppetScopeDocument.Factory.newInstance();
		        pScopeDoc.addNewPuppetScope();
		        PuppetScope pScope = pScopeDoc.getPuppetScope();
				for(int n = 0; n < ivalues.length; n++) {
					// Extracting the name of the operation
					ElectronicServiceInterfaceDefinition_Impl iface = (ElectronicServiceInterfaceDefinition_Impl) ivalues[n];
					pScope.addNewMethod().setNameMethod(iface.getIdentifier());
				}
		        importedNode = scopeElement.getDomNode().getOwnerDocument().importNode(
		                pScope.getDomNode(), true);
		        scopeElement.getDomNode().appendChild(importedNode);
			}
			
			
        }

        return scopeElement;
    }

    /*
     * Appends and returns a "QualifyingCondition" element to the
     * GuaranteeTermType
     */
    protected XmlObject generateQualifyingCondition(GuaranteeTermType guaranteeTerm) {
        Node importedNode;
        XmlObject qcElement = guaranteeTerm.addNewQualifyingCondition();

        return qcElement;
    }

    /*
     * Appends and returns a BusinessValueListType element to the
     * GuaranteeTermType
     */
    protected BusinessValueListType generateBusinessValueList(GuaranteeTermType guaranteeTerm) {
        Node importedNode;
        BusinessValueListType bvlElement = guaranteeTerm
                .addNewBusinessValueList();

        return bvlElement;
    }
    
    
    public abstract TermCompositorType generate();
}
